
/*
 * File GlobalSubsumption.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#include "Debug/RuntimeStatistics.hpp"

#include "Lib/Metaiterators.hpp"
#include "Lib/SharedSet.hpp"
#include "Lib/TimeCounter.hpp"
#include "Lib/VirtualIterator.hpp"

#include "Kernel/Clause.hpp"
#include "Kernel/Grounder.hpp"
#include "Kernel/Inference.hpp"

#include "Indexing/Index.hpp"
#include "Indexing/IndexManager.hpp"
#include "Indexing/GroundingIndex.hpp"

#include "SAT/SATClause.hpp"
#include "SAT/SATSolver.hpp"
#include "SAT/SATInference.hpp"

#include "Saturation/SaturationAlgorithm.hpp"

#include "Shell/Options.hpp"
#include "Shell/Statistics.hpp"

#include "GlobalSubsumption.hpp"
#include "Saturation/Splitter.hpp"

#undef LOGGING
#define LOGGING 0

namespace Inferences {

using namespace Lib;
using namespace Kernel;
using namespace Indexing;
using namespace Saturation;

void GlobalSubsumption::attach(SaturationAlgorithm* salg)
{
  CALL("GlobalSubsumption::attach");

  ASS(!_index);

  ForwardSimplificationEngine::attach(salg);
  _index=static_cast<GroundingIndex*>(
	  _salg->getIndexManager()->request(GLOBAL_SUBSUMPTION_INDEX) );

  if (_salg->getOptions().globalSubsumptionAvatarAssumptions() == Options::GlobalSubsumptionAvatarAssumptions::FULL_MODEL) {
    _splitter=_salg->getSplitter();
  } else {
    _splitter = 0;
  }
}

void GlobalSubsumption::detach()
{
  CALL("GlobalSubsumption::detach");

  _index=0;
  _salg->getIndexManager()->release(GLOBAL_SUBSUMPTION_INDEX);
  ForwardSimplificationEngine::detach();
}

Clause* GlobalSubsumption::perform(Clause* cl, Stack<Unit*>& prems)
{
  CALL("GlobalSubsumption::perform/2");

  TimeCounter tc(TC_GLOBAL_SUBSUMPTION);

  if(cl->color()==COLOR_LEFT) {
    return cl;
  }
   
  if(!_splittingAssumps && cl->splits() && cl->splits()->size()!=0) {
    return cl;
  }
  
  Grounder& grounder = _index->getGrounder();
  
  
  static SATLiteralStack plits;
  plits.reset();
  
  
  
  static SATLiteralStack assumps;
  assumps.reset();
  
  
  static DHMap<SATLiteral,Literal*> lookup;
  lookup.reset();
    
  
  
  grounder.groundNonProp(cl, plits, false);
  
  unsigned clen = plits.size();    
  for (unsigned i = 0; i < clen; i++) {
    lookup.insert(plits[i],(*cl)[i]);
    assumps.push(plits[i].opposite());
  }
    
  
  
  
  if (cl->splits() && cl->splits()->size()!=0) {
    ASS(_splittingAssumps);
    
    SplitSet::Iterator sit(*cl->splits());
    while(sit.hasNext()) {
      SplitLevel l = sit.next();      
      unsigned var = splitLevelToVar(l);
                
      plits.push(SATLiteral(var,false)); 
      if (!_splitter) {
        assumps.push(SATLiteral(var,true)); 
      }
    }
  }
  
  
  if (_splitter) {
    ASS(_splittingAssumps);
    
    SplitLevel bound = _splitter->splitLevelBound();
    for (SplitLevel lev = 0; lev < bound; lev++) {
      if (_splitter->splitLevelActive(lev)) {
        unsigned var = splitLevelToVar(lev);
        assumps.push(SATLiteral(var,true)); 
      }
    }
  }
  
  SATSolverWithAssumptions& solver = _index->getSolver();
  
  
  
  
  

  
  SATClause* scl = SATClause::fromStack(plits);
  SATInference* inf = new FOConversionInference(cl);
  scl->setInference(inf);
  solver.addClause(scl);

  
  SATSolver::Status res = solver.solveUnderAssumptions(assumps, _uprOnly, true);

  if (res == SATSolver::UNSATISFIABLE) { 
    
    

    const SATLiteralStack& failed = solver.failedAssumptions();

    if (failed.size() < assumps.size()) {
      
      const SATLiteralStack& failedFinal = _explicitMinim ? solver.explicitlyMinimizedFailedAssumptions(_uprOnly,_randomizeMinim) : failed;

      static LiteralStack survivors;
      survivors.reset();

      static Set<SATLiteral> splitAssumps;
      splitAssumps.reset();

      for (unsigned i = 0; i < failedFinal.size(); i++) {
        SATLiteral olit = failedFinal[i].opposite(); 

        Literal* lit;
        if (lookup.find(olit,lit)) { 
          survivors.push(lit);
        } else { 
          splitAssumps.insert(olit);
        }
      }

      

      
      if (survivors.size() < clen) {
        RSTAT_MCTR_INC("global_subsumption_by_number_of_removed_literals",clen-survivors.size());

        SATClause* ref = solver.getRefutation();

        prems.reset();
        prems.push(cl);
                
        SATInference::collectFilteredFOPremises(ref, prems,
          
          
          [this,cl] (SATClause* prem) {

            
            if (prem->inference()->getType() == SATInference::ASSUMPTION) {
              ASS_EQ(prem->size(),1);
              return false;
            }

            
            unsigned prem_sz = prem->size();
            for (unsigned i = 0; i < prem_sz; i++ ) {
              SATLiteral lit = (*prem)[i];
              SplitLevel lev;
              if (isSplitLevelVar(lit.var(),lev)) {
                ASS(lit.isNegative());
                if (!splitAssumps.contains(lit)) {
                  return false;
                }
              }
            }
            return true;
          } );
        
        UnitList* premList = 0;
        Stack<Unit*>::Iterator it(prems);
        while (it.hasNext()) {
          Unit* us = it.next();
          UnitList::push(us, premList);
        }
        
        SATClauseList* satPremises = solver.getRefutationPremiseList();

        Inference* inf = satPremises ? 
             new InferenceFromSatRefutation(Inference::GLOBAL_SUBSUMPTION, premList, satPremises, failedFinal) :
             new InferenceMany(Inference::GLOBAL_SUBSUMPTION, premList);

        Clause* replacement = Clause::fromIterator(LiteralStack::BottomFirstIterator(survivors),cl->inputType(), inf);
        replacement->setAge(cl->age());
        
        
                
        env.statistics->globalSubsumption++;
        ASS_L(replacement->length(), clen);
        
        return replacement;       
      }                  
    }
  }

  return cl;
}

struct GlobalSubsumption::Unit2ClFn
{
  DECL_RETURN_TYPE(Clause*);
  OWN_RETURN_TYPE operator() (Unit* us) {
    return us->asClause();
  }
};

bool GlobalSubsumption::perform(Clause* cl, Clause*& replacement, ClauseIterator& premises)
{
  CALL("GlobalSubsumption::perform/3");

  static Stack<Unit*> prems;
  
  Clause* newCl = perform(cl,prems);
  if(newCl==cl) {
    return false;
  }
    
  Stack<Unit*>::BottomFirstIterator it(prems);

  replacement = newCl;
  premises = pvi( getMappingIterator(it, Unit2ClFn()) );
  return true;
}

}
