
/*
 * File MinisatInterfacing.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#include "MinisatInterfacing.hpp"

#include "Lib/ScopedLet.hpp"

#include "Lib/DArray.hpp"

namespace SAT
{

using namespace Shell;  
using namespace Lib;  
  
using namespace Minisat;
  
MinisatInterfacing::MinisatInterfacing(const Shell::Options& opts, bool generateProofs):
  _status(SATISFIABLE)
{
  CALL("MinisatInterfacing::MinisatInterfacing");
   
  
  
}
  
void MinisatInterfacing::ensureVarCount(unsigned newVarCnt)
{
  CALL("MinisatInterfacing::ensureVarCount");
  
  while(_solver.nVars() < (int)newVarCnt) {
    _solver.newVar();
  }
}

unsigned MinisatInterfacing::newVar() 
{
  CALL("MinisatInterfacing::newVar");
  
  return minisatVar2Vampire(_solver.newVar());
}

SATSolver::Status MinisatInterfacing::solveUnderAssumptions(const SATLiteralStack& assumps, unsigned conflictCountLimit, bool)
{
  CALL("MinisatInterfacing::solveUnderAssumptions");

  ASS(!hasAssumptions());

  
  SATLiteralStack::ConstIterator it(assumps);
  while (it.hasNext()) {
    _assumptions.push(vampireLit2Minisat(it.next()));
  }

  solveModuloAssumptionsAndSetStatus(conflictCountLimit);

  if (_status == SATSolver::UNSATISFIABLE) {
    
    _failedAssumptionBuffer.reset();
    Minisat::LSet& conflict = _solver.conflict;
    for (int i = 0; i < conflict.size(); i++) {
      _failedAssumptionBuffer.push(minisatLit2Vampire(conflict[i]).opposite());
    }
  }

  _assumptions.clear();

  return _status;
}

void MinisatInterfacing::solveModuloAssumptionsAndSetStatus(unsigned conflictCountLimit) 
{
  CALL("MinisatInterfacing::solveModuloAssumptionsAndSetStatus");
  
  
    
  _solver.setConfBudget(conflictCountLimit); 
  lbool res = _solver.solveLimited(_assumptions);
  
  if (res == l_True) {
    _status = SATISFIABLE;
  } else if (res == l_False) {
    _status = UNSATISFIABLE;    
  } else {
    _status = UNKNOWN;
  }
}

void MinisatInterfacing::addClause(SATClause* cl)
{
  CALL("MinisatInterfacing::addClause");
  
  
  PrimitiveProofRecordingSATSolver::addClause(cl);
  
  
  
  ASS_EQ(_assumptions.size(),0);
                
  static vec<Lit> mcl;
  mcl.clear();
    
  unsigned clen=cl->length();
  for(unsigned i=0;i<clen;i++) {
    SATLiteral l = (*cl)[i];
    mcl.push(vampireLit2Minisat(l));
  }

  _solver.addClause(mcl);
}

SATSolver::Status MinisatInterfacing::solve(unsigned conflictCountLimit)
{
  CALL("MinisatInterfacing::solve");
  
  solveModuloAssumptionsAndSetStatus(conflictCountLimit);
  return _status;
}

void MinisatInterfacing::addAssumption(SATLiteral lit) 
{
  CALL("MinisatInterfacing::addAssumption");
  
  _assumptions.push(vampireLit2Minisat(lit));
}

SATSolver::VarAssignment MinisatInterfacing::getAssignment(unsigned var) 
{
  CALL("MinisatInterfacing::getAssignment");
	ASS_EQ(_status, SATISFIABLE);  
	ASS_G(var,0); ASS_LE(var,(unsigned)_solver.nVars());
  lbool res;
    
  Minisat::Var mvar = vampireVar2Minisat(var);  
  if (mvar < _solver.model.size()) {  
    if ((res = _solver.modelValue(mvar)) == l_True) {
      return TRUE;
    } else if (res == l_False) {    
      return FALSE;
    } else {              
      ASSERTION_VIOLATION;
      return NOT_KNOWN;
    }
  } else { 
    return DONT_CARE;
  }
}

bool MinisatInterfacing::isZeroImplied(unsigned var)
{
  CALL("MinisatInterfacing::isZeroImplied");
  ASS_G(var,0); ASS_LE(var,(unsigned)_solver.nVars());
  
 
  return _solver.value(vampireVar2Minisat(var)) != l_Undef;
}

void MinisatInterfacing::collectZeroImplied(SATLiteralStack& acc)
{
  CALL("MinisatInterfacing::collectZeroImplied");
  
  
  
  
  
  for (Minisat::Var v = 0; v < _solver.nVars(); v++) {
    lbool val = _solver.value(v);
    if (val != l_Undef) { 
      
      
      acc.push(minisatLit2Vampire(mkLit(v,val == l_False)));
    }
  }        
}

SATClause* MinisatInterfacing::getZeroImpliedCertificate(unsigned)
{
  CALL("MinisatInterfacing::getZeroImpliedCertificate");
  
  
  
 
  
  return 0;
}

SATClauseList* MinisatInterfacing::minimizePremiseList(SATClauseList* premises, SATLiteralStack& assumps)
{
  CALL("MinisatInterfacing::minimizePremiseList");

  Minisat::Solver solver;

  static DHMap<int,SATClause*> var2prem;
  var2prem.reset();

  static vec<Lit> ass; 
  ass.clear();

  int cl_no = 0;

  SATClauseList* it= premises;
  while(it) {
    
    var2prem.insert(cl_no,it->head());

    
    ass.push(mkLit(cl_no)); 

    
    ALWAYS(solver.newVar() == cl_no);

    cl_no++;
    it=it->tail();
  }

  
  int offset = cl_no; 

  
  int curmax = cl_no;

  
  cl_no = 0;
  it= premises;
  while(it) {
    SATClause* cl = it->head();

    static vec<Lit> mcl;
    mcl.clear();

    
    unsigned clen=cl->length();
    for(unsigned i=0;i<clen;i++) {
      SATLiteral l = (*cl)[i];
      int var = offset + l.var();

      
      while (var >= curmax) {
        solver.newVar();
        curmax++;
      }

      mcl.push(mkLit(var,l.isNegative()));
    }

    
    mcl.push(mkLit(cl_no,true)); 

    solver.addClause(mcl);

    cl_no++;
    it=it->tail();
  }

  
  SATLiteralStack::Iterator ait(assumps);
  while (ait.hasNext()) {
    SATLiteral l = ait.next();
    int var = offset + l.var();

    ASS_L(var,curmax);

    ass.push(mkLit(var,l.isNegative()));
  }

  
  ALWAYS(!solver.solve(ass)); 

#if 0 
  
  ass.clear();
  Minisat::LSet& conflict = solver.conflict;
  for (int i = 0; i < conflict.size(); i++) {
    ass.push(~conflict[i]);
  }

  Lit dummy = mkLit(offset); 
  
  for (int i = 0; i < ass.size(); i++) {
    Lit tmp = ass[i];
    ass[i] = dummy;
    if (solver.solve(ass)) { 
      ass[i] = tmp;
    }
  }

  SATClauseList* result = SATClauseList::empty();
  for (int i = 0; i < ass.size(); i++) {
    int v = var(ass[i]);
    SATClause* cl;
    if (var2prem.find(v,cl)) {
      SATClauseList::push(cl,result);
    }
  }
#else

  SATClauseList* result = SATClauseList::empty();

  
  Minisat::LSet& conflict = solver.conflict;
  for (int i = 0; i < conflict.size(); i++) {
    int v = var(conflict[i]);

    SATClause* cl;

    if (var2prem.find(v,cl)) {
      SATClauseList::push(cl,result);
    } 
  }
#endif

  return result;
}

void MinisatInterfacing::interpolateViaAssumptions(unsigned maxVar, const SATClauseStack& first, const SATClauseStack& second, SATClauseStack& result)
{
  CALL("MinisatInterfacing::interpolateViaAssumptions");

  Minisat::Solver solver_first;
  Minisat::Solver solver_second;

  
  
  
  
  
  
  for(unsigned v = 0; v <= maxVar; v++) { 
    solver_first.newVar();
    solver_second.newVar();
  }

  DArray<bool> varOfFirst;
  varOfFirst.expand(maxVar+1,false);

  vec<Lit> tmp;

  
  SATClauseStack::ConstIterator it1(first);
  while(it1.hasNext()) {
    SATClause* cl = it1.next();

    unsigned clen=cl->length();
    for(unsigned i=0;i<clen;i++) {
      SATLiteral l = (*cl)[i];
      varOfFirst[l.var()] = true;
      tmp.push(mkLit(l.var(),l.isNegative()));
    }

    solver_first.addClause(tmp);
    tmp.clear();
  }

  
  SATClauseStack::ConstIterator it2(second);
  while(it2.hasNext()) {
    SATClause* cl = it2.next();

    unsigned clen=cl->length();
    for(unsigned i=0;i<clen;i++) {
      SATLiteral l = (*cl)[i];
      tmp.push(mkLit(l.var(),l.isNegative()));
    }

    solver_second.addClause(tmp);
    tmp.clear();
  }

  SATLiteralStack vlits;

  
  while (solver_first.solve()) {
    
    for (int i = 1; i <= (int)maxVar; i++) {
      if (varOfFirst[i]) {
        tmp.push(mkLit(i,solver_first.model[i]==l_False));
      }
    }

    NEVER(solver_second.solve(tmp));
    tmp.clear();

    
    LSet& conflict = solver_second.conflict;
    for (int i = 0; i < conflict.size(); i++) {
      Lit l = conflict[i];

      tmp.push(l);
      vlits.push(SATLiteral(var(l),sign(l) ? 0 : 1));
    }

    solver_first.addClause(tmp);
    tmp.clear();

    result.push(SATClause::fromStack(vlits));
    vlits.reset();
  }
}


} 

