
/*
 * File MinisatInterfacingNewSimp.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#include "Forwards.hpp"

#include "MinisatInterfacingNewSimp.hpp"

#include "Lib/System.hpp"
#include "Lib/ScopedLet.hpp"
#include "Lib/Environment.hpp"
#include "Shell/Statistics.hpp"

#include "Minisat/core/SolverTypes.h"
#include <limits>

namespace SAT
{

using namespace Shell;  
using namespace Lib;  
  
using namespace Minisat;

const unsigned MinisatInterfacingNewSimp::VAR_MAX = std::numeric_limits<Minisat::Var>::max() / 2;
  
MinisatInterfacingNewSimp::MinisatInterfacingNewSimp(const Shell::Options& opts, bool generateProofs):
  _status(SATISFIABLE)
{
  CALL("MinisatInterfacingNewSimp::MinisatInterfacingNewSimp");
   
  
  
  
  limitMemory(opts.memoryLimit()*1);
}

void MinisatInterfacingNewSimp::reportMinisatOutOfMemory() {
  env.beginOutput();
  reportSpiderStatus('m');
  env.out() << "Minisat ran out of memory" << endl;
  if(env.statistics) {
    env.statistics->print(env.out());
  }
#if VDEBUG
  Debug::Tracer::printStack(env.out());
#endif
  env.endOutput();
  System::terminateImmediately(1);
}

void MinisatInterfacingNewSimp::ensureVarCount(unsigned newVarCnt)
{
  CALL("MinisatInterfacingNewSimp::ensureVarCount");
  
  try{
    while(_solver.nVars() < (int)newVarCnt) {
      _solver.newVar();
    }
  } catch (Minisat::OutOfMemoryException&){
    reportMinisatOutOfMemory();
  }
}

unsigned MinisatInterfacingNewSimp::newVar() 
{
  CALL("MinisatInterfacingNewSimp::ensureVarCount");
  
  return minisatVar2Vampire(_solver.newVar());
}

SATSolver::Status MinisatInterfacingNewSimp::solveUnderAssumptions(const SATLiteralStack& assumps, unsigned conflictCountLimit, bool)
{
  CALL("MinisatInterfacingNewSimp::solveUnderAssumptions");

  ASS(!hasAssumptions());

  
  SATLiteralStack::ConstIterator it(assumps);
  while (it.hasNext()) {
    _assumptions.push(vampireLit2Minisat(it.next()));
  }

  solveModuloAssumptionsAndSetStatus(conflictCountLimit);

  if (_status == SATSolver::UNSATISFIABLE) {
    
    _failedAssumptionBuffer.reset();
    Minisat::LSet& conflict = _solver.conflict;
    for (int i = 0; i < conflict.size(); i++) {
      _failedAssumptionBuffer.push(minisatLit2Vampire(conflict[i]).opposite());
    }
  }

  _assumptions.clear();

  return _status;
}

void MinisatInterfacingNewSimp::solveModuloAssumptionsAndSetStatus(unsigned conflictCountLimit) 
{
  CALL("MinisatInterfacingNewSimp::solveModuloAssumptionsAndSetStatus");
  
  
   
  try{
    
    

    _solver.setConfBudget(conflictCountLimit); 
    lbool res = _solver.solveLimited(_assumptions,true,true);

    
  
    if (res == l_True) {
      _status = SATISFIABLE;
    } else if (res == l_False) {
      _status = UNSATISFIABLE;
    } else {
      _status = UNKNOWN;
    }
  }catch(Minisat::OutOfMemoryException&){
    reportMinisatOutOfMemory();
  }
}

void MinisatInterfacingNewSimp::addClause(SATClause* cl)
{
  CALL("MinisatInterfacingNewSimp::addClause");

  
  
  ASS_EQ(_assumptions.size(),0);

  try {
    static vec<Lit> mcl;
    mcl.clear();
    
    unsigned clen=cl->length();
    for(unsigned i=0;i<clen;i++) {
      SATLiteral l = (*cl)[i];
      mcl.push(vampireLit2Minisat(l));
    }
    _solver.addClause(mcl);
  } catch (Minisat::OutOfMemoryException&){
      reportMinisatOutOfMemory();
  }
}

SATSolver::Status MinisatInterfacingNewSimp::solve(unsigned conflictCountLimit)
{
  CALL("MinisatInterfacingNewSimp::solve");
  
  solveModuloAssumptionsAndSetStatus(conflictCountLimit);
  return _status;
}

void MinisatInterfacingNewSimp::addAssumption(SATLiteral lit) 
{
  CALL("MinisatInterfacingNewSimp::addAssumption");
  
  _assumptions.push(vampireLit2Minisat(lit));
}

SATSolver::VarAssignment MinisatInterfacingNewSimp::getAssignment(unsigned var) 
{
  CALL("MinisatInterfacingNewSimp::getAssignment");
	ASS_EQ(_status, SATISFIABLE);  
	ASS_G(var,0); ASS_LE(var,(unsigned)_solver.nVars());
  lbool res;
    
  Minisat::Var mvar = vampireVar2Minisat(var);  
  if (mvar < _solver.model.size()) {  
    if ((res = _solver.modelValue(mvar)) == l_True) {
      return TRUE;
    } else if (res == l_False) {    
      return FALSE;
    } else {              
      ASSERTION_VIOLATION;
      return NOT_KNOWN;
    }
  } else { 
    return DONT_CARE;
  }
}

bool MinisatInterfacingNewSimp::isZeroImplied(unsigned var)
{
  CALL("MinisatInterfacingNewSimp::isZeroImplied");
  ASS_G(var,0); ASS_LE(var,(unsigned)_solver.nVars());
  
 
  return _solver.value(vampireVar2Minisat(var)) != l_Undef;
}

void MinisatInterfacingNewSimp::collectZeroImplied(SATLiteralStack& acc)
{
  CALL("MinisatInterfacingNewSimp::collectZeroImplied");
  
  
  
  
  
  for (Minisat::Var v = 0; v < _solver.nVars(); v++) {
    lbool val = _solver.value(v);
    if (val != l_Undef) { 
      
      
      acc.push(minisatLit2Vampire(mkLit(v,val == l_False)));
    }
  }        
}

SATClause* MinisatInterfacingNewSimp::getZeroImpliedCertificate(unsigned)
{
  CALL("MinisatInterfacingNewSimp::getZeroImpliedCertificate");
  
  
  
 
  
  return 0;
}

} 

