
/*
 * File MinisatInterfacingNewSimp.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */

#ifndef __MinisatInterfacingNewSimp__
#define __MinisatInterfacingNewSimp__


#include <signal.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "SATSolver.hpp"
#include "SATLiteral.hpp"
#include "SATClause.hpp"

#include "Minisat/simp/SimpSolver.h"

namespace SAT{

class MinisatInterfacingNewSimp : public SATSolverWithAssumptions
{
public:
  CLASS_NAME(MinisatInterfacingNewSimp);
  USE_ALLOCATOR(MinisatInterfacingNewSimp);
  
  static const unsigned VAR_MAX;

	MinisatInterfacingNewSimp(const Shell::Options& opts, bool generateProofs=false);

 
  virtual void addClause(SATClause* cl) override;
  
 
  virtual void simplify() override {
    CALL("MinisatInterfacingNewSimp::simplify");
    _solver.simplify();
  }

  virtual Status solve(unsigned conflictCountLimit) override;
  
 
  virtual VarAssignment getAssignment(unsigned var) override;

 
  virtual bool isZeroImplied(unsigned var) override;
 
  virtual void collectZeroImplied(SATLiteralStack& acc) override;
 
  virtual SATClause* getZeroImpliedCertificate(unsigned var) override;

  virtual void ensureVarCount(unsigned newVarCnt) override;
  
  virtual unsigned newVar() override;
  
  virtual void suggestPolarity(unsigned var, unsigned pol) override {
    
    bool mpol = pol ? false : true; 
    _solver.suggestPolarity(vampireVar2Minisat(var),mpol);
  }
  
 
  virtual void addAssumption(SATLiteral lit) override;
  
  virtual void retractAllAssumptions() override {
    _assumptions.clear();
    _status = UNKNOWN;
  };
  
  virtual bool hasAssumptions() const override {
    return (_assumptions.size() > 0);
  };


  virtual void recordSource(unsigned satlitvar, Literal* lit) override {
    
  };
  
  Status solveUnderAssumptions(const SATLiteralStack& assumps, unsigned conflictCountLimit, bool) override;

  virtual SATClause* getRefutation() override { ASSERTION_VIOLATION; }

  static void reportMinisatOutOfMemory();

protected:
  void solveModuloAssumptionsAndSetStatus(unsigned conflictCountLimit = UINT_MAX);
  
  Minisat::Var vampireVar2Minisat(unsigned vvar) {
    ASS_G(vvar,0); ASS_LE(vvar,(unsigned)_solver.nVars());
    return (vvar-1);
  }
  
  unsigned minisatVar2Vampire(Minisat::Var mvar) {
    return (unsigned)(mvar+1);
  }
  
  const Minisat::Lit vampireLit2Minisat(SATLiteral vlit) {
    return Minisat::mkLit(vampireVar2Minisat(vlit.var()),vlit.isNegative()); 
  }
  
 
  const SATLiteral minisatLit2Vampire(Minisat::Lit mlit) {
    return SATLiteral(minisatVar2Vampire(Minisat::var(mlit)),Minisat::sign(mlit) ? 0 : 1);            
  }
  
private:
  Status _status;
  Minisat::vec<Minisat::Lit> _assumptions;  
  Minisat::SimpSolver _solver;

  
  static void limitMemory(uint64_t max_mem_mb)
  {
      
      if (max_mem_mb != 0){
          rlim_t new_mem_lim = (rlim_t)max_mem_mb * 1024*1024;
          rlimit rl;
          getrlimit(RLIMIT_AS, &rl);
          if (rl.rlim_max == RLIM_INFINITY || new_mem_lim < rl.rlim_max){
              rl.rlim_cur = new_mem_lim;
              if (setrlimit(RLIMIT_AS, &rl) == -1)
                  printf("WARNING! Could not set resource limit: Virtual memory.\n");
          }
      }
  }
};

}

 #endif
