
/*
 * File SubexpressionIterator.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __SubexpressionIterator__
#define __SubexpressionIterator__

#include "Kernel/Formula.hpp"
#include "Kernel/Term.hpp"
#include "Lib/VirtualIterator.hpp"

namespace Shell {
  using namespace Lib;
  using namespace Kernel;

  
  class SubexpressionIterator { 
    public:
      CLASS_NAME(SubexpressionIterator);
      USE_ALLOCATOR(SubexpressionIterator);
     
      class Expression {
        public:
          enum Tag {
            FORMULA,
            TERM
          };

          Expression(Formula* f): Expression(f, 1) {}
          Expression(Formula* f, int polarity): _tag(FORMULA), _formula(f), _polarity(polarity) {}
          Expression(TermList ts): Expression(ts, 1) {}
          Expression(TermList ts, int polarity): _tag(TERM), _term(ts), _polarity(polarity) {}
          Expression(Term* t): Expression(TermList(t)) {}
          Expression(Term* t, int polarity): Expression(TermList(t), polarity) {}

          bool isTerm() { return _tag == TERM; }
          TermList getTerm() { ASS(isTerm()); return _term; }

          bool isFormula() { return _tag == FORMULA; }
          Formula* getFormula() { ASS(isFormula()); return _formula; }

          int getPolarity() { return _polarity; }

          friend class SubexpressionIterator;

        private:
          Tag _tag;
          union {
            Formula* _formula;
            TermList _term;
          };
          int _polarity;
      };

      bool hasNext();
      Expression next();

      SubexpressionIterator(Formula* f);
      SubexpressionIterator(FormulaList* fs);
      SubexpressionIterator(Term* t);
      SubexpressionIterator(TermList ts);

    private:
      Stack<Expression> _subexpressions;
  };

  class FoolAwareSubformulaIterator : public IteratorCore<Formula*> {
    public:
      FoolAwareSubformulaIterator(Formula* f): _sei(f) {}
      FoolAwareSubformulaIterator(FormulaList* fs): _sei(fs) {}
      FoolAwareSubformulaIterator(Term* t): _sei(t) {}
      FoolAwareSubformulaIterator(TermList ts): _sei(ts) {}

      bool hasNext() {
        CALL("FoolAwareSubformulaIterator::hasNext");
        while (_sei.hasNext()) {
          SubexpressionIterator::Expression expression = _sei.next();
          if (expression.isFormula()) {
            _next = expression.getFormula();
            _polarity = expression.getPolarity();
            return true;
          }
        }
        return false;
      }
      Formula* next() {
        int dummy;
        return next(dummy);
      }
      Formula* next(int& polarity) {
        CALL("FoolAwareSubformulaIterator::next(int&)");
        ASS(_next);
        polarity = _polarity;
        return _next;
      }

    private:
      SubexpressionIterator _sei;
      Formula* _next;
      int _polarity;
  };

  class FoolAwareSubtermIterator : public IteratorCore<TermList> {
    public:
      FoolAwareSubtermIterator(Formula* f): _sei(f) {}
      FoolAwareSubtermIterator(FormulaList* fs): _sei(fs) {}
      FoolAwareSubtermIterator(Term* t): _sei(t) {}
      FoolAwareSubtermIterator(TermList ts): _sei(ts) {}

      bool hasNext() {
        CALL("FoolAwareSubtermIterator::hasNext");
        while (_sei.hasNext()) {
          SubexpressionIterator::Expression expression = _sei.next();
          if (expression.isTerm()) {
            _next = expression.getTerm();
            return true;
          }
        }
        return false;
      }
      TermList next() {
        return _next;
      }

    private:
      SubexpressionIterator _sei;
      TermList _next;
  };
}

#endif 
