
/*
 * File TPTPPrinter.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#include <sstream>

#include "Lib/DHMap.hpp"
#include "Lib/Environment.hpp"
#include "Lib/SharedSet.hpp"

#include "Kernel/Signature.hpp"
#include "Kernel/Clause.hpp"
#include "Kernel/SortHelper.hpp"

#include "Parse/TPTP.hpp"

#include "Kernel/Term.hpp"
#include "Kernel/Inference.hpp"
#include "Kernel/Unit.hpp"
#include "Kernel/Formula.hpp"
#include "Kernel/FormulaUnit.hpp"
#include "Kernel/Clause.hpp"

#include "TPTPPrinter.hpp"

#include "Forwards.hpp"

namespace Shell
{
TPTPPrinter::TPTPPrinter(ostream* tgtStream)
: _tgtStream(tgtStream), _headersPrinted(false)
{
  CALL("TPTPPrinter::TPTPPrinter");
}

void TPTPPrinter::print(Unit* u)
{
  CALL("TPTPPrinter::print");

  vstring body = getBodyStr(u);

  beginOutput();
  ensureHeadersPrinted(u);
  printTffWrapper(u, body);
  endOutput();
}

void TPTPPrinter::printAsClaim(vstring name, Unit* u)
{
  CALL("TPTPPrinter::printAsClaim");
  vstring body = getBodyStr(u);

  beginOutput();

  ensureHeadersPrinted(u);

  tgt() << "tff(" << name << ", claim, " << body << ")." << endl;

  endOutput();
}

vstring TPTPPrinter::getBodyStr(Unit* u)
{
  CALL("TPTPPrinter::getBodyStr");

  vostringstream res;

  typedef DHMap<unsigned,unsigned> SortMap;
  static SortMap varSorts;
  varSorts.reset();
  SortHelper::collectVariableSorts(u, varSorts);

  if(u->isClause()) {
    SortMap::Iterator vit(varSorts);
    bool quantified = vit.hasNext();
    if(quantified) {
      res << "![";
      while(vit.hasNext()) {
	unsigned var, varSort;
	vit.next(var, varSort);

	res << 'X' << var;
	if(varSort!=Sorts::SRT_DEFAULT) {
	  res << " : " << env.sorts->sortName(varSort);
	}
	if(vit.hasNext()) {
	  res << ',';
	}
      }
      res << "]: (";
    }

    Clause* cl = static_cast<Clause*>(u);
    Clause::Iterator cit(*cl);
    if(!cit.hasNext()) {
      res << "$false";
    }
    while(cit.hasNext()) {
      Literal* lit = cit.next();
      res << lit->toString();
      if(cit.hasNext()) {
	res << " | ";
      }
    }

    if(quantified) {
      res << ')';
    }

    if(!cl->noSplits()) {
      SplitSet::Iterator sit(*cl->splits());
      while(sit.hasNext()) {
	SplitLevel split = sit.next();
	res << " | " << "$splitLevel" << split;
      }
    }
  }
  else {
    return static_cast<FormulaUnit*>(u)->formula()->toString();
  }
  return res.str();
}

void TPTPPrinter::printTffWrapper(Unit* u, vstring bodyStr)
{
  CALL("TPTPPrinter::printTffWrapper");

  tgt() << "tff(";
  vstring unitName;
  if(Parse::TPTP::findAxiomName(u, unitName)) {
    tgt() << unitName;
  }
  else {
    tgt() << "u_" << u->number();
  }
  tgt() << ", ";
  switch(u->inputType()) {
  case Unit::AXIOM:
    tgt() << "axiom"; break;
  case Unit::ASSUMPTION:
    tgt() << "hypothesis"; break;
  case Unit::CONJECTURE:
    tgt() << "conjecture"; break;
  case Unit::NEGATED_CONJECTURE:
    tgt() << "negated_conjecture"; break;
  case Unit::CLAIM:
    tgt() << "claim"; break;
  case Unit::EXTENSIONALITY_AXIOM:
    tgt() << "extensionality"; break;
  default:
     ASSERTION_VIOLATION;
  }
  tgt() << ", " << endl << "    " << bodyStr << " )." << endl;
}

void TPTPPrinter::outputSymbolTypeDefinitions(unsigned symNumber, bool function)
{
  CALL("TPTPPrinter::outputSymbolTypeDefinitions");

  Signature::Symbol* sym = function ?
      env.signature->getFunction(symNumber) : env.signature->getPredicate(symNumber);
  BaseType* type = function ? static_cast<BaseType*>(sym->fnType()) : static_cast<BaseType*>(sym->predType());

  if(type->isAllDefault()) {
    return;
  }
  if(function && theory->isInterpretedConstant(symNumber)) { return; }

  if(sym->interpreted()) {
    Interpretation interp = static_cast<Signature::InterpretedSymbol*>(sym)->getInterpretation();
    switch(interp) {
    case Theory::INT_SUCCESSOR:
    case Theory::INT_ABS:
    case Theory::INT_DIVIDES:
      
      break;
    default:
      return;
    }
  }

  tgt() << "tff(" << (function ? "func" : "pred") << "_def_" << symNumber << ",type, "
      << sym->name() << ": ";

  unsigned arity = sym->arity();
  if(arity>0) {
    tgt() << "(";
    for(unsigned i=0; i<arity; i++) {
      if(i>0) {
	tgt() << " * ";
      }
      tgt() << env.sorts->sortName(type->arg(i));
    }
    tgt() << ") > ";
  }
  if(function) {
    tgt() << env.sorts->sortName(sym->fnType()->result());
  }
  else {
    tgt() << "$o";
  }
  tgt() << " )." << endl;
}

void TPTPPrinter::ensureNecesarySorts()
{
  CALL("TPTPPrinter::ensureNecesarySorts");
  if (_headersPrinted) {
    return;
  }
  unsigned i;
  List<unsigned> *_usedSorts(0);
  BaseType* type;
  Signature::Symbol* sym;
  unsigned sorts = env.sorts->sorts();
  
  for (i = 0; i < env.signature->functions(); i++) {
    sym = env.signature->getFunction(i);
    type = static_cast<BaseType*>(sym->fnType());
    unsigned arity = sym->arity();
    if (arity > 0) {
      for (unsigned i = 0; i < arity; i++) {
	if( _usedSorts->member(type->arg(i))==false)
	 _usedSorts = _usedSorts->cons(type->arg(i));
      }
    }
  }
  
  for (i = 0; i < env.signature->predicates(); i++) {
    sym = env.signature->getPredicate(i);
    type = static_cast<BaseType*>(sym->predType());
    unsigned arity = sym->arity();
    if (arity > 0) {
      for (unsigned i = 0; i < arity; i++) {
	if( _usedSorts->member(type->arg(i))==false)
		  _usedSorts = _usedSorts->cons(type->arg(i));
      }
    }
  }
  
  for (i = Sorts::FIRST_USER_SORT; i < sorts; i++) {
    if (_usedSorts->member(i))
      tgt() << "tff(sort_def_" << i << ",type, " << env.sorts->sortName(i)
            	      << ": $tType" << " )." << endl;

  }
}

void TPTPPrinter::ensureHeadersPrinted(Unit* u)
{
  CALL("TPTPPrinter::ensureHeadersPrinted");

  if(_headersPrinted) {
    return;
  }

  ensureNecesarySorts();

  unsigned funs = env.signature->functions();
  for(unsigned i=0; i<funs; i++) {
    outputSymbolTypeDefinitions(i, true);
  }
  unsigned preds = env.signature->predicates();
  for(unsigned i=1; i<preds; i++) {
    outputSymbolTypeDefinitions(i, false);
  }

  _headersPrinted = true;
}

ostream& TPTPPrinter::tgt()
{
  CALL("TPTPPrinter::tgt");

  if(_tgtStream) {
    return *_tgtStream;
  }
  else {
    return env.out();
  }
}

void TPTPPrinter::beginOutput()
{
  CALL("TPTPPrinter::beginOutput");

  if(!_tgtStream) { env.beginOutput(); }
}

void TPTPPrinter::endOutput()
{
  CALL("TPTPPrinter::endOutput");

  if(!_tgtStream) { env.endOutput(); }
}

vstring TPTPPrinter::toString(const Formula* formula)
{
  CALL("TPTPPrinter::toString(const Formula*)");
  static vstring names [] =
    { "", " & ", " | ", " => ", " <=> ", " <~> ",
      "~", "!", "?", "$term", "$false", "$true", "", ""};
  ASS_EQ(sizeof(names)/sizeof(vstring), NOCONN+1);

  vstring res;

  
  typedef pair<Connective,const Formula*> Todo;
  Stack<Todo> stack;

  stack.push(make_pair(NOCONN,formula));

  while (stack.isNonEmpty()) {
    Todo todo = stack.pop();

    
    res += names[todo.first];

    const Formula* f = todo.second;

    if (!f) {
      res += ")";
      continue;
    }

    Connective c = f->connective();

    switch (c) {
    case LITERAL: {
      vstring result = f->literal()->toString();
      if (f->literal()->isEquality()) {
        res += "(" + result + ")";
      } else {
        res += result;
      }
      continue;
    }
    case AND:
    case OR:
      {
        
        

        const FormulaList* fs = f->args();
        res += "(";
        stack.push(make_pair(NOCONN,nullptr)); 
        while (FormulaList::isNonEmpty(fs)) {
          const Formula* arg = fs->head();
          fs = fs->tail();
          
          stack.push(make_pair(FormulaList::isNonEmpty(fs) ? c : NOCONN,arg));
        }

        continue;
      }
    case IMP:
    case IFF:
    case XOR:
      

      res += "(";

      stack.push(make_pair(NOCONN,nullptr)); 

      stack.push(make_pair(c,f->right())); 

      stack.push(make_pair(NOCONN,f->left())); 

      continue;

    case NOT:
      res += "(";

      stack.push(make_pair(NOCONN,nullptr)); 

      stack.push(make_pair(c,f->uarg()));

      continue;

    case FORALL:
    case EXISTS:
      {
        vstring result = vstring("(") + names[c] + "[";
        bool needsComma = false;
        Formula::VarList::Iterator vs(f->vars());
        Formula::SortList::Iterator ss(f->sorts());
        bool hasSorts = f->sorts();

        while (vs.hasNext()) {
          int var = vs.next();

          if (needsComma) {
            result += ", ";
          }
          result += 'X';
          result += Int::toString(var);
          unsigned t;
          if (hasSorts) {
            ASS(ss.hasNext());
            t = ss.next();
            if (t != Sorts::SRT_DEFAULT) {
              result += " : " + env.sorts->sortName(t);
            }
          } else if (SortHelper::tryGetVariableSort(var, const_cast<Formula*>(f),
              t) && t != Sorts::SRT_DEFAULT) {
            result += " : " + env.sorts->sortName(t);
          }
          needsComma = true;
        }
        res += result + "] : (";

        stack.push(make_pair(NOCONN,nullptr));
        stack.push(make_pair(NOCONN,nullptr)); 

        stack.push(make_pair(NOCONN,f->qarg()));

        continue;
      }

    case BOOL_TERM:
      res += f->getBooleanTerm().toString();

      continue;

    case FALSE:
    case TRUE:
      res += names[c];

      continue;
    default:
      ASSERTION_VIOLATION;
    }
  }
  return res;
}

vstring TPTPPrinter::toString (const Unit* unit)
{
  CALL("TPTPPrinter::toString(const Unit*)");



  vstring prefix;
  vstring main = "";

  bool negate_formula = false;
  vstring kind;
  switch (unit->inputType()) {
  case Unit::ASSUMPTION:
    kind = "hypothesis";
    break;

  case Unit::CONJECTURE:
    if(unit->isClause()) {
      kind = "negated_conjecture";
    }
    else {
      negate_formula = true;
      kind = "conjecture";
    }
    break;

  case Unit::EXTENSIONALITY_AXIOM:
    kind = "extensionality";
    break;

  case Unit::NEGATED_CONJECTURE:
    kind = "negated_conjecture";
    break;

  default:
    kind = "axiom";
    break;
  }

  if (unit->isClause()) {
    prefix = "cnf";
    main = static_cast<const Clause*>(unit)->toTPTPString();
  }
  else {
    prefix = "tff";
    const Formula* f = static_cast<const FormulaUnit*>(unit)->formula();
    if(negate_formula) {
      Formula* quant=Formula::quantify(const_cast<Formula*>(f));
      if(quant->connective()==NOT) {
	ASS_EQ(quant, f);
	main = toString(quant->uarg());
      }
      else {
	Formula* neg=new NegatedFormula(quant);
	main = toString(neg);
	neg->destroy();
      }
      if(quant!=f) {
	ASS_EQ(quant->connective(),FORALL);
	static_cast<QuantifiedFormula*>(quant)->vars()->destroy();
	quant->destroy();
      }
    }
    else {
      main = toString(f);
    }
  }

  vstring unitName;
  if(!Parse::TPTP::findAxiomName(unit, unitName)) {
    unitName="u" + Int::toString(unit->number());
  }

  return prefix + "(" + unitName + "," + kind + ",\n"
    + "    " + main + ").\n";
}


vstring TPTPPrinter::toString(const Term* t){
  NOT_IMPLEMENTED;
}

vstring TPTPPrinter::toString(const Literal* l){
  NOT_IMPLEMENTED;
}

}
